------------------------------------------------------------------------
-- Config
------------------------------------------------------------------------

local ADDON_NAME, NPG = ...

local AceGUI = LibStub("AceGUI-3.0")
assert(AceGUI, ADDON_NAME .. " Requires AceGUI-3.0")

NPG.Config = {}
do
   local DEFAULT_LAYOUT_STRING = 
   [[
-- Welcome to the extremely sophisticated (and user-friendly) configuration screen.
-- Configuration is currently available through text only.
-- Lines starting with '--' are comment lines.
-- There is no documentation. Hopefully the commands are self-explanatory!

-- If this is your first time using NPG, you should hit the Accept button
-- to load these default settings.

-- Then simply go stand somewhere where you can see nameplates
-- (e.g. target dummies, or town with friendly nameplates visible)
-- and mess around with the settings below until you achieve something desirable.
   
-- grid(x, y) - define nameplategrid to be x columns wide and y rows tall
grid(5,3)

-- size(width, height) - define size of each nameplate cell
size(90, 36)

-- space(width, height) - define amount of empty space between each cell
space(5, 10)


-- border(style) - define border style for cells. accepted values are 1 - 3
border(1)

-- bordersize(size) - set size for nameplate cell border
bordersize(16)

-- border(r, g, b, a) - set color and alpha for nameplate cell border
bordercolor(0.5, 0.5, 0.5, 1)

-------------------------

-- display possible raid target icon at top of nameplate cell
icon("raid")
:point("TOP", "CENTER", 0, 5)
:size(20, 20)

-- display name of nameplate unit in the top left corner
text("%name")
:point("TOPLEFT", 3, -3)
:width(70)
:height(10)

-- display three red question marks if the GUID of a nameplate is not known
text("???")
:point("TOPLEFT", 3, -15)
:color(1,0,0)
:func(function()
   visible(not GUID)
end)

-- display health percentage in bottom right corner
text("%healthpct")
:point("BOTTOMRIGHT", 0, 2)

-- display aura #1
-- (aura# icons are special thingies that display the icon of a tracked debuff (defined later)
--  The icon will turn deep red-ish when the aura is about to drop (<30% remaining)
--  and disappears completely if the debuff is not active)
icon("aura1")
:point("BOTTOMLEFT", 3, 3)
:size(16, 16)

-- display aura #2
icon("aura2")
:point("BOTTOMLEFT", 20, 3)
:size(16, 16)

-- #3
icon("aura3")
:point("BOTTOMLEFT", 37, 3)
:size(16, 16)

-- define auras to track (required for the aura# icons) based on currently played class
if CLASS == "PRIEST" then
   auras("Shadow Word: Pain", "Vampiric Touch")
elseif CLASS == "WARLOCK" then
   auras("Agony", 146739, "Unstable Affliction")
-- some auras require spellIDs to work, for whatever reason. 146739 = corruption
elseif CLASS == "DRUID" then
   auras("Moonfire", "Sunfire", "Insect Swarm")
end

binds{
   -- binds for all characters
   all={
      -- quality of life "quick" binds for raid targets 8 = skull, 7 = cross, ... 1 = star
      ["SHIFT-1"] = "rt8",
      ["SHIFT-2"] = "rt7",
      ["SHIFT-3"] = "rt6",
      ["SHIFT-4"] = "rt5",
      ["CTRL-1"] = "rt4",
      ["CTRL-2"] = "rt3",
      ["CTRL-3"] = "rt2",
      ["CTRL-4"] = "rt1",
   },
   
   -- binds for priest class
   PRIEST = {
      ["BUTTON1"] = "Shadow Word: Pain",
      ["BUTTON2"] = "Vampiric Touch",
      ["BUTTON3"] = "Mind Flay",
   },
   
   -- warlock
   WARLOCK = {
      ["1"] = "Agony",
      ["2"] = "Corruption",
      ["3"] = "Unstable Affliction",
   },
   
   -- etc.
   -- name of class needs to be uppercase, 
   -- with spaces removed, e.g. WARRIOR, DEATHKNIGHT
}
   ]]

   local DEFAULT_DB = {
      FirstTime = true,
      LayoutString = DEFAULT_LAYOUT_STRING,
      ContainerPoint = "CENTER",
      ContainerX = 0,
      ContainerY = 0,
   }
   
   local DB
   local ConfigFrame
   local f = CreateFrame("frame")
   local DragOverlay = CreateFrame("Frame")
   local LoadInCombat = false
   
   
   do
      local f = DragOverlay
      f:SetFrameStrata("HIGH")
      f.container = NPG.Core:GetContainerFrame()
      
      f:SetAllPoints(f.container)
      f.BGTexture = f:CreateTexture()
      f.BGTexture:SetTexture(0, 0.7, 0.7, 0.5)
      f.BGTexture:SetAllPoints()
      
      f.Text = f:CreateFontString()
      f.Text:SetFontObject("GameFontNormal")
      f.Text:SetAllPoints()
      f.Text:SetText("Click & Drag to move\10Should not (= can not) be placed over other UI elements\10\10Right click to hide this overlay")
      
      f:RegisterForDrag("LeftButton")
      f:EnableMouse(true)
      f:SetScript("OnDragStart", function() f.container:StartMoving() end)
      f:SetScript("OnDragStop", function() 
         f.container:StopMovingOrSizing() 
         local point, _, _, offX, offY = f.container:GetPoint(1)
         DB.ContainerPoint = point
         DB.ContainerX = offX
         DB.ContainerY = offY
      end)
      
      f:SetScript("OnMouseDown", function(s, b)
         if b == "RightButton" then s:Hide() end
      end)
      f:Hide()
   end
   
   local function EnableDrag()
      DragOverlay.container:SetMovable(true)
      DragOverlay:Show()
   end
   
   local function DisableDrag()
      DragOverlay.container:SetMovable(false)
      DragOverlay:Hide()
   end
   
   local function CloseFrame(widget)
      ConfigFrame = nil
      AceGUI:Release(widget)
      DisableDrag()
   end
   
   local function LoadLayoutString(silent)
      if not InCombatLockdown() then 
         LoadInCombat = false
         NPG.Frames:LoadLayoutString(DB.LayoutString)
      else 
         LoadInCombat = true
         if not silent then NPG._addon.print("Layout will be loaded once you leave combat.") end
      end
   end
   
   local function SetupFrame()
      if ConfigFrame then return end
      
      if DB.FirstTime then EnableDrag() end
      local f = AceGUI:Create("Window")
      f:SetTitle(ADDON_NAME)
      f:SetLayout("Flow")
      f:EnableResize(false)
      f:SetWidth(600)
      f:SetHeight(520)
      f:SetPoint("LEFT", 50, 0)
   
      local w
   
      w = AceGUI:Create("MultiLineEditBox")
      w:SetMaxLetters(0)
      w:SetLabel(nil)
      w:SetNumLines(30)
      w:DisableButton(true)
      w:SetFullWidth(true)
      w:SetFullHeight(false)
      w:SetText(DB.LayoutString)
      f:AddChild(w)
      
      local eb = w
      
      w = AceGUI:Create("Button")
      w:SetText("Reset to default")
      w:SetWidth(150)
      w:SetHeight(32)
      w:SetCallback("OnClick", function() eb:SetText(DEFAULT_LAYOUT_STRING) end)
      f:AddChild(w)
      
      w = AceGUI:Create("Label")
      w:SetText("")
      w:SetWidth(100)
      f:AddChild(w)
      
      w = AceGUI:Create("Button")
      w:SetText("Move Frame")
      w:SetWidth(150)
      w:SetHeight(32)
      w:SetCallback("OnClick", function(w) if DragOverlay:IsVisible() then DisableDrag() else EnableDrag() end end)
      f:AddChild(w)
      
      w = AceGUI:Create("Button")
      w:SetText("Accept")
      w:SetWidth(150)
      w:SetHeight(32)
      w:SetDisabled(not DB.FirstTime)
      w:SetCallback("OnClick", function() 
         DB.LayoutString = eb:GetText()
         LoadLayoutString()
         DB.FirstTime = false
      end)
      f:AddChild(w)
      
      eb:SetCallback("OnTextChanged", function() w:SetDisabled(false) end)
   
      f:SetCallback("OnClose", CloseFrame)
      ConfigFrame = f
   end
   
   
   
   function f:ADDON_LOADED(addon)
      if addon == ADDON_NAME then
         DB = NamePlateGrid_DB
         
         if not DB then
            DB = DEFAULT_DB -- should probably copy the table instead of using the default table, but, meh!
            NamePlateGrid_DB = DB
         end
         
         if DB.FirstTime then
            NPG._addon.print("Type \124cffff0000/npg\124r for first-time setup")
         else
            NPG.Frames:LoadLayoutString(DB.LayoutString)
         end
         
         NPG.Core:GetContainerFrame():SetPoint(DB.ContainerPoint, nil, DB.ContainerPoint, DB.ContainerX, DB.ContainerY)
      end
   end
   
   function f:PLAYER_SPECIALIZATION_CHANGED()
      LoadLayoutString(1)
   end
   
   function f:PLAYER_REGEN_ENABLED()
      if LoadInCombat then
         LoadLayoutString()
         LoadInCombat = false
      end
   end
   
   NPG._addon.RegisterEvents(f)

   _G.SLASH_NAMEPLATEGRID1 = "/npg"
   _G.SLASH_NAMEPLATEGRID2 = "/nameplategrid"
   SlashCmdList.NAMEPLATEGRID = SetupFrame
end